# Importing the libraries
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from sklearn import metrics
from sklearn.model_selection import train_test_split
from sklearn.impute import SimpleImputer
from sklearn.linear_model import LinearRegression
from sklearn.metrics import r2_score
from sklearn.svm import SVR

def calculateGradientLinerRegressionAndSVR(x_list,y_list):
    # -------- Preprocessing Input data -------------------------------------------------------
    # Importing the dataset
    X = np.array(x_list)
    Y = np.array(y_list)

    # Splitting the dataset into the Training set and Test set
    X_train, X_test, y_train, y_test = train_test_split(X, Y, test_size = 0.25, random_state = 0)

    # Reshape (Gives a new shape to an array without changing its data. | 1D Array to 2D Array)
    X_train= X_train.reshape(-1, 1)
    y_train= y_train.reshape(-1, 1)
    X_test= X_test.reshape(-1, 1)
    y_test= y_test.reshape(-1, 1)

    # -------- Build the Model ***Linear Regression(LR)*****---------------------------------
    # Fitting Simple Linear Regression to the Training set
    regressor_LR = LinearRegression()
    regressor_LR.fit(X_train,y_train) #training the algorithm
    
    # Predicting the Test set result
    y_pred = regressor_LR.predict(X_test)

    # Get Gradient
    gradient_LR = regressor_LR.coef_

    # Calulate r-Squared
    r_squared_linear_regression = r2_score(y_test, y_pred)

    # Visualising the Training set result
    plt.scatter(X_train, y_train, color = 'red')
    plt.plot(X_train, regressor_LR.predict(X_train), color = 'blue')
    plt.title("Linear Regression")
    plt.xlabel('Allocated weight for question' )
    plt.ylabel('Spend time per question')
    # plt.show()

    # -------- Build the Model ***Support Vector Regression(SVR)*****-------------------------------------
    # Fitting Support Vector Regression to the Training set
    regressor_SVR = SVR(kernel = 'linear')
    regressor_SVR.fit(X_train,y_train) #training the algorithm
    
    # Predicting the Test set result
    y_pred = regressor_SVR.predict(X_test)

    # Get Gradient
    gradient_SVR = regressor_SVR.coef_

    # Calulate r-Squared
    r_squared_support_vector_regression = r2_score(y_test, y_pred)
    

    # Visualising the Training set result
    plt.scatter(X_train, y_train, color = 'red')
    plt.plot(X_train, regressor_SVR.predict(X_train), color = 'blue')
    plt.title("Support Vector Regression")
    plt.xlabel('Allocated weight for question' )
    plt.ylabel('Spend time per question')
    # plt.show()

    # Compare and add best gradient to the list
    if r_squared_linear_regression >= r_squared_support_vector_regression:
      print("linear_regression_Gardient = ",gradient_LR)
      print("linear_regression_Gardient = ",gradient_SVR)
      print("r_squared_linear_regression = ",r_squared_linear_regression)
      print("r_squared_support_vector_regression = ",r_squared_support_vector_regression)
      return gradient_LR
    else:
      print("support_vector_regression_Gardient = ",gradient_LR)
      print("support_vector_regression_Gardient = ",gradient_SVR)
      print("r_squared_linear_regression = ",r_squared_linear_regression)
      print("r_squared_support_vector_regression = ",r_squared_support_vector_regression)
      return gradient_SVR