import React, { Component } from 'react'
import { Link, withRouter, BrowserRouter as Router, Route } from 'react-router-dom';
import axios from 'axios'
import constant from '../service/constant';
import disableBrowserBackButton from 'disable-browser-back-navigation';
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import { faSignOutAlt } from '@fortawesome/free-solid-svg-icons';
import Chart from 'react-apexcharts'

class StudentDashBoard extends Component {

  constructor(props) {
    super(props);
    this.progressListeHandler = this.progressListeHandler.bind(this);
    this.state = {
      SystemUser: this.props.location.state.SystemUser,
      ProgressDataList: [],
      progressLable: '',
      //Start --- charts values
      optionsMixedChart: {
        chart: { id: "basic-bar", toolbar: { show: false } },
        plotOptions: { bar: { columnWidth: "50%", endingShape: "arrow" } },
        stroke: { width: [4, 0, 0] },
        xaxis: { categories: [] },
        markers: { size: 6, strokeWidth: 3, fillOpacity: 0, strokeOpacity: 0, hover: { size: 8 } },
        yaxis: { tickAmount: 5, min: -10, max: 10, floating: false, labels: { formatter: (val) => { return val.toFixed(3) }, } }
      },
      seriesMixedChart: [{ name: "Learning progress", type: "line", data: [] }],
      optionsRadial: {
        plotOptions: {
          radialBar: {
            startAngle: -135,
            endAngle: 225,
            hollow: { margin: 0, size: "70%", background: "#fff", image: undefined, imageOffsetX: 0, imageOffsetY: 0, position: "front", dropShadow: { enabled: true, top: 3, left: 0, blur: 4, opacity: 0.24 } },
            track: { background: "#fff", strokeWidth: "67%", margin: 0, dropShadow: { enabled: true, top: -3, left: 0, blur: 4, opacity: 0.35 } },
            dataLabels: { showOn: "always", name: { offsetY: -20, show: true, color: "#888", fontSize: "13px" }, value: { formatter: function (val) { return val.toFixed(3); }, color: "#111", fontSize: "30px", show: true } }
          }
        },
        fill: { type: "gradient", gradient: { shade: "dark", type: "horizontal", shadeIntensity: 0.5, gradientToColors: ["#ABE5A1"], inverseColors: true, opacityFrom: 1, opacityTo: 1, stops: [0, 100] } },
        stroke: { lineCap: "round" },
        labels: ["Avarage Learning Progress"]
      },
      seriesRadial: [100],
      optionsBar: {
        chart: {
          stacked: true,
          stackType: "100%",
          toolbar: {
            show: false
          }
        },
        plotOptions: {
          bar: {
            horizontal: true
          }
        },
        dataLabels: {
          dropShadow: {
            enabled: true
          }
        },
        stroke: {
          width: 0
        },
        xaxis: {
          categories: ["Fav Color"],
          labels: {
            show: false
          },
          axisBorder: {
            show: false
          },
          axisTicks: {
            show: false
          }
        },
        fill: {
          opacity: 1,
          type: "gradient",
          gradient: {
            shade: "dark",
            type: "vertical",
            shadeIntensity: 0.35,
            gradientToColors: undefined,
            inverseColors: false,
            opacityFrom: 0.85,
            opacityTo: 0.85,
            stops: [90, 0, 100]
          }
        },

        legend: {
          position: "bottom",
          horizontalAlign: "right"
        }
      },
      seriesBar: [
        {
          name: "blue",
          data: [32]
        },
        {
          name: "green",
          data: [41]
        },
        {
          name: "yellow",
          data: [12]
        },
        {
          name: "red",
          data: [65]
        }
      ]
      //End --- charts values
    }
  }


  componentWillMount() {
    // disableBrowserBackButton();

    var sid = this.state.SystemUser.systemUser.id;

    console.log(this.state.optionsMixedChart.xaxis.categories);
    axios.get(constant() + '/getprogress/' + sid + '/')
      .then(res => {
        //set data list and categories(label) to the dashboard at first time
        var dataNew = res.data[0].learningProgresList
        var categoriesNew = res.data[0].paperLabelList
        var totalLearningRate = 0;
        var seriesRadialNew = 0;
        var seriesRadialNewListType = [];


        //Calulate total Learning progress
        res.data[0].learningProgresList.map(function (learningRate, index) {
          totalLearningRate = totalLearningRate + learningRate;
        })

        //Calulate Avarage Learning progress
        seriesRadialNew = totalLearningRate / res.data[0].learningProgresList.length;
        seriesRadialNewListType.push(seriesRadialNew);//seriesRadial should List type

        this.setState({
          ProgressDataList: res.data,
          optionsMixedChart: { xaxis: { categories: categoriesNew } }, //Object property(categories) Assign 
          seriesMixedChart: [{ data: dataNew }], //Object property(data) Assign
          seriesRadial: seriesRadialNewListType,
        });

        console.log(this.state.ProgressDataList);

      }).catch(err => {
        console.log(err);
      });

  }

  listHandler = (e) => {
    const state = this.state
    state[e.target.name] = e.target.value;
    this.setState(state)
  }

  paperSearch = (e) => {
    e.preventDefault()
    this.props.history.push({
      pathname: '/Student2',
      state: {
        SystemUser: this.state.SystemUser,
      }
    });
  }

  paperView = (e) => {
    e.preventDefault()
    this.props.history.push({
      pathname: '/Papers',
      state: {
        SystemUser: this.state.SystemUser,
      }
    });
  }

  logOutButtonHandler = () => {
    this.props.history.push("/")
  }

  progressListeHandler = (e) => {
    const state = this.state
    state[e.target.name] = e.target.value;
    this.setState(state) 

    var dataNew = []
    var categoriesNew = []
    var seriesRadialNewListType = [];

    //Calulate total Learning progress
    this.state.ProgressDataList.map(function (Progress, index) {
      
      if(Progress.subTopic === e.target.value){

        //set data list and categories(label) to the dashboard at first time
        dataNew = Progress.learningProgresList
        categoriesNew = Progress.paperLabelList
        var totalLearningRate = 0;
        var seriesRadialNew = 0;
        seriesRadialNewListType = [];

        //Calulate total Learning progress
        Progress.learningProgresList.map(function (learningRate, index) {
          totalLearningRate = totalLearningRate + learningRate;
        })

        //Calulate Avarage Learning progress
        seriesRadialNew = totalLearningRate / Progress.learningProgresList.length;
        seriesRadialNewListType.push(seriesRadialNew);//seriesRadial should List type
      }
    });

    this.setState({
      optionsMixedChart: { xaxis: { categories: categoriesNew } }, //Object property(categories) Assign 
      seriesMixedChart: [{ data: dataNew }], //Object property(data) Assign
      seriesRadial: seriesRadialNewListType,
    });

  }

  render() {

    return (

      <div className="container">
        <div className="panel panel-default">
          <div className="panel-heading">
            <div className="row">
              <div className="col-md-10">
                <h3 className="panel-title">
                  Student Dashboard
                </h3>
              </div>
              <div className="col-md-2-right">
                <button type="submit" className="btn btn btn-outline-danger" onClick={this.logOutButtonHandler.bind(this)}><FontAwesomeIcon icon={faSignOutAlt} />  Logout</button>
              </div>
            </div>
          </div><br />
        </div>
        <div className="panel-body">
          <button type="submit" className="btn btn-outline-success btn-lg btn-block" onClick={this.paperSearch.bind(this)}>Paper Search</button><br />
          <button type="submit" className="btn btn-outline-info btn-lg btn-block" onClick={this.paperView.bind(this)}>View My Papers</button>
        </div>
        <div>
          <div className="app">
            <div className="row">
              <div className="col-md-6 mixed-chart">
                <Chart
                  options={this.state.optionsMixedChart}
                  series={this.state.seriesMixedChart}
                  type="line"
                  width="500"
                />
              </div>

              <div className="col-md-3 radial-chart">
                <Chart
                  options={this.state.optionsRadial}
                  series={this.state.seriesRadial}
                  type="radialBar"
                  width="280"
                />
              </div>
              <div className="col-md-3">
                <label>Select a paper series:</label>
                <select className="form-control" name="progressLable" onChange={this.progressListeHandler} value={this.state.progressLable}>
                  {
                    this.state.ProgressDataList.map(sub => {
                      return (
                        <option key={sub.subTopic} value={sub.subTopic}>{sub.subTopic} </option>
                      )
                    })
                  }
                </select>
              </div>
            </div>
            {/* <div className="row">
              <div className="col percentage-chart">
                <Chart
                  options={this.state.optionsBar}
                  height={140}
                  series={this.state.seriesBar}
                  type="bar"
                  width={500}
                />
              </div>

              <p className="col">
                <button onClick={this.updateCharts}>Update!</button>
              </p>
            </div> */}
          </div>
        </div>
      </div>
    )
  }
}

export default StudentDashBoard;
