package com.cdap_2020_027.elearning.ws.service;

import com.cdap_2020_027.elearning.ws.model.LoginResponse;
import com.cdap_2020_027.elearning.ws.model.PaperResult;
import com.cdap_2020_027.elearning.ws.model.PaperResultFinal;
import com.cdap_2020_027.elearning.ws.model.PaperResultStudent;
import com.cdap_2020_027.elearning.ws.model.ProgressData;
import com.cdap_2020_027.elearning.ws.model.Question;
import com.cdap_2020_027.elearning.ws.model.QuestionResult;
import com.cdap_2020_027.elearning.ws.model.SearchList;
import com.cdap_2020_027.elearning.ws.model.StudentAnswer;
import com.cdap_2020_027.elearning.ws.repository.PaperResultFinalRepository;
import com.cdap_2020_027.elearning.ws.repository.PaperResultStudentRepository;
import com.cdap_2020_027.elearning.ws.repository.QuestionRepository;
import com.cdap_2020_027.elearning.ws.util.FetchDataIntegration;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.util.UUID;
import com.fasterxml.jackson.databind.ObjectMapper;

/**
 *
 * @author IT17119122 Liyanage I.M
 */
@Service
public class QuestionServiceImpl {

    @Autowired
    QuestionRepository questionRepository;

    @Autowired
    PaperResultStudentRepository paperResultStudentRepository;

    @Autowired
    PaperResultFinalRepository paperResultFinalRepository;

    public SearchList getSearchList() {
        List<String> areaList = questionRepository.getQuestionAreas();
        List<String> topicList = questionRepository.getQuestionTopics();
        List<Question> SubTopicList = questionRepository.getQuestionSubTopics();

//        List<SubTopicDto> SubTopicDtoList = new ArrayList<>();
//        int i=1;
//        //create Value, lable paire for the react-selecter
//        for (String SubTopic : SubTopicList) {
//            SubTopicDto subTopicDto = new SubTopicDto();
//            subTopicDto.setId(i);
//            subTopicDto.setLabel(SubTopic);
//
//            SubTopicDtoList.add(subTopicDto);
//            i++;
//        }
        SearchList searchList = new SearchList();
        searchList.setAreaList(areaList);
        searchList.setTopicList(topicList);
        searchList.setSubTopicList(SubTopicList);

        return searchList;
    }

    public List<Question> getQuestionList(String area, String topic, List<String> subtopic) {
        return questionRepository.getQuestionList(area, topic, subtopic);
    }

    public LoginResponse questionPaperResult(PaperResult paperResult) {

        //get paper result details and save in a Paper Result final table
        PaperResultFinal paperResultFinal = new PaperResultFinal();

        paperResultFinal.setSid(paperResult.getLoginResponse().getSystemUser().getId());
        paperResultFinal.setPaperNo(paperResult.getPaperNo());
        paperResultFinal.setTotalQuestionNo(paperResult.getTotalQuestionNo());
        paperResultFinal.setCorrectQuestionNo(paperResult.getCorrectQuestionNo());
        paperResultFinal.setAverageScore(paperResult.getAverageScore());
        paperResultFinal.setGrade(paperResult.getGrade());
        paperResultFinal.setStatus(paperResult.getStatus());
        paperResultFinal.setArea(paperResult.getArea());
        paperResultFinal.setTopic(paperResult.getTopic());

        paperResultFinal.setDate(paperResult.getDate());

        //get subtopic array list
        List<String> subtopic = paperResult.getSubtopic();
        String subTopicString = ""; //initialize
        //Concat first item of the array
        subTopicString = subTopicString.concat(subtopic.get(0));
        //SubTopic list convert to comma seperated string
        int s = 0;
        for (String subTopic : paperResult.getSubtopic()) {
            if (s != 0) {
                subTopicString = subTopicString.concat("," + subTopic);
            }
            s++;
        }

        paperResultFinal.setSubtopic(subTopicString);

        //Unique id Assign
        long uniqueId = 0;
        if (paperResult.getPaperNo() == 1) {
            //generate uniq id for the new paper collection
            uniqueId = UUID.randomUUID().hashCode();

            paperResultFinal.setUniqueId(Long.toString(uniqueId));
        } else {
            //same previous uniqeID set to the object
            paperResultFinal.setUniqueId(paperResult.getUniqueId());
        }

        PaperResultFinal paperResultFinalNew = paperResultFinalRepository.save(paperResultFinal);

        //Set the flag for identify contine quation paper
        if (paperResult.getPaperNo() == 1) {
            paperResultFinalRepository.updateFlag("ACTIVE", paperResultFinalNew.getId());
        } else {
            //set current paper ACTIVATE
            paperResultFinalRepository.updateFlag("ACTIVE", paperResultFinalNew.getId());
            //set previous(LAST) paper DEACTIVATE
            paperResultFinalRepository.updateFlagPrevious("DEACTIVE", paperResult.getPreviouPaperId());
        }

        //save result obj one by one in to paperResultStudent table
        for (StudentAnswer studentAnswer : paperResult.getStudentAnswerList()) {
            PaperResultStudent paperResultStudent = new PaperResultStudent();

            if (paperResult.getPaperNo() == 1) {
            paperResultStudent.setUniqueId(Long.toString(uniqueId));
            } else {
                //same previous uniqeID set to the object
                paperResultStudent.setUniqueId(paperResult.getUniqueId());
            }
            
            paperResultStudent.setFinalPaperId(paperResultFinalNew.getId());
            paperResultStudent.setPaperNo(paperResult.getPaperNo());
            paperResultStudent.setQuestionId(studentAnswer.getQuestionId());
            paperResultStudent.setWeight(studentAnswer.getWeight());
            paperResultStudent.setStudentAnswer(Integer.parseInt(studentAnswer.getStudentAnswer()));
            paperResultStudent.setCorrectAnswer(studentAnswer.getCorrectAnswer());
            paperResultStudent.setTime(studentAnswer.getTime());

            paperResultStudentRepository.save(paperResultStudent);
        }
        //return the current logged user
        return paperResult.getLoginResponse();
    }

    public List<PaperResultFinal> getPapers(int id) {
        return paperResultFinalRepository.getPapers(id);
    }

    public QuestionResult getPaper(int id) {
        //Find the paper
        PaperResultFinal paperResultFinal = paperResultFinalRepository.getPaper(id);
        //get all question belong to the paper
        List<PaperResultStudent> answerList = paperResultStudentRepository.getPaper(paperResultFinal.getId());

        List<Question> questionList = new ArrayList<>();

        for (PaperResultStudent paperResultStudent : answerList) {
            Question question = new Question();
            question = questionRepository.getQuestion(paperResultStudent.getQuestionId());
            questionList.add(question);
        }

        //Set Question Result and Question list to new objct for pass to frontend
        QuestionResult questionResult = new QuestionResult();
        questionResult.setAnswerList(answerList);
        questionResult.setQuestionList(questionList);

        return questionResult;
    }

    //Flask web service integration method
    public List<ProgressData> getLearningProgress(int sid) throws IOException {

        List<Object[]> uniqueIdList = paperResultFinalRepository.getUniqueIds(sid);

        List<ProgressData> ProgressDataList = new ArrayList<ProgressData>();
        for (Object[] obj : uniqueIdList) {
            //Http Request class Object
            FetchDataIntegration fetchData = new FetchDataIntegration();
            //Call the flask web service throuh the FetchDataIntegration class
//            String jsonString = fetchData.fetch("/progress/" + Integer.parseInt(paperResultFinal.getUniqueId()));
            String jsonString = fetchData.fetch("/progress/" + obj[0]);
            ObjectMapper mapper = new ObjectMapper();
            ProgressData progressData = mapper.readValue(jsonString, ProgressData.class); // Json String convert to the ProgressData class

            //Add Paper label list using LearningProgresList size  
            ArrayList<String> PaperLabelList = new ArrayList<>();
            for (int i = 1; i <= progressData.getLearningProgresList().size(); i++) {
                String paperStr = "Paper " + Integer.toString(i);
                PaperLabelList.add(paperStr);
            }
            progressData.setPaperLabelList(PaperLabelList); //Set the PaperLabelList to progressData Object
            progressData.setSubTopic(obj[1]+" _ "+obj[0]);

            ProgressDataList.add(progressData); //add the progressData Object to ProgressDataList 
        }

        return ProgressDataList;
    }

//        if (uniqueId == "first") {
//
//        }
//        ProgressData obj = progressData;
//
//        System.out.println(progressData.getLearningProgresList());
//        return MinistryV3;
}
