package com.cdap_2020_027.elearning.ws.service;

import com.cdap_2020_027.elearning.ws.model.SystemUser;
import com.cdap_2020_027.elearning.ws.repository.SystemUserRepository;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;

import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 *
 * @author (IT17119122 ** Liyanage I.M)
 */
@Service
public class SystemUserServiceImpl {

    @Autowired
    SystemUserRepository systemUserRepository;

    @Autowired
    private JavaMailSender javaMailSender;

    //Get standeard character UTF_8 UNICODE
    private static final Charset UTF_8 = StandardCharsets.UTF_8;

    public SystemUser findByUsernameandPassword(String username, String password) {
        
        //Passwordconvert to MD5(Encription)
        byte[] md5InBytes = this.digest(password.getBytes(UTF_8)); //password convert to MD5 byte code
        String passwordMd5 = bytesToHex(md5InBytes); //byte code encripted password convert to string MD5
        
        SystemUser systemUser = systemUserRepository.findByUsernameandPassword(username, passwordMd5);
        return systemUser;
    }

    public void saveUser(SystemUser systemUser) {

        String password = systemUser.getPassword();
        //Passwordconvert to MD5(Encription)
        byte[] md5InBytes = this.digest(password.getBytes(UTF_8));
        //Set SEncripted password
        systemUser.setPassword(bytesToHex(md5InBytes));
        systemUserRepository.save(systemUser);
        this.sendMail(systemUser);

    }

    public void sendMail(SystemUser systemUser) {
        // Create a default MimeMessage object.
        SimpleMailMessage message = new SimpleMailMessage();
        message.setFrom("inusham45@gmail.com");
        message.setTo(systemUser.getEmail());
        // Set Subject: header field
        message.setSubject("E-Learning platform register confirmation");
        // Now set the actual message
        message.setText("Congratulations! , You are successfully register to the E-Learning platform");
        // Send message
        javaMailSender.send(message);
        System.out.println("Sent message successfully....");
    }

    //encripted input to MD5 byte code
    private static byte[] digest(byte[] input) {
        MessageDigest md;
        try {
            md = MessageDigest.getInstance("MD5");
        } catch (NoSuchAlgorithmException e) {
            throw new IllegalArgumentException(e);
        }
        byte[] result = md.digest(input);
        return result;
    }

    //MD5 byte code convert to String
    private static String bytesToHex(byte[] bytes) {
        StringBuilder sb = new StringBuilder();
        for (byte b : bytes) {
            sb.append(String.format("%02x", b));
        }
        return sb.toString();
    }

}
