package com.cdap_2020_027.elearning.ws.controller;

import com.cdap_2020_027.elearning.ws.model.LoginResponse;
import com.cdap_2020_027.elearning.ws.model.PaperResult;
import com.cdap_2020_027.elearning.ws.model.PaperResultFinal;
import com.cdap_2020_027.elearning.ws.model.ProgressData;
import com.cdap_2020_027.elearning.ws.model.Question;
import com.cdap_2020_027.elearning.ws.model.QuestionResult;
import com.cdap_2020_027.elearning.ws.model.SearchList;
import com.cdap_2020_027.elearning.ws.service.QuestionServiceImpl;
import com.cdap_2020_027.elearning.ws.service.SystemUserServiceImpl;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

/**
 *
 * @author IT17119122 Liyanage I.M
 */
@RestController
@CrossOrigin
public class QuestionController {

    @Autowired
    QuestionServiceImpl questionServiceImpl;

    @Autowired
    SystemUserServiceImpl systemUserServiceImpl;

    // get area / topic /sub topic Details
    @RequestMapping(value = "/searchlist", method = RequestMethod.GET)
    public ResponseEntity<SearchList> getAreas() {
        try {
            SearchList searchList = questionServiceImpl.getSearchList();
            return new ResponseEntity<>(searchList, HttpStatus.OK);
        } catch (Exception ex) {
            System.out.println(ex);
            return new ResponseEntity<>(null, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    // get qustion paper Details 
    @RequestMapping(value = "/questions/{area}/{topic}", method = RequestMethod.POST)
    public ResponseEntity<List<Question>> getQuestionList(@PathVariable(value = "area") String area,
            @PathVariable(value = "topic") String topic, @RequestBody List<String> subtopic) {
        try {
            System.out.println(subtopic);
            List<Question> questionList = questionServiceImpl.getQuestionList(area, topic, subtopic);
            return new ResponseEntity<>(questionList, HttpStatus.OK);
        } catch (Exception ex) {
            System.out.println(ex);
            return new ResponseEntity<>(null, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    // Save qustion paper result Details 
    @RequestMapping(value = "/paperresult", method = RequestMethod.POST)
    public ResponseEntity<LoginResponse> questionPaperResult(@RequestBody PaperResult paperResult) {
        try {
            LoginResponse loginResponse = questionServiceImpl.questionPaperResult(paperResult);
            return new ResponseEntity<>(loginResponse, HttpStatus.OK);
        } catch (Exception ex) {
            System.out.println(ex);
            return new ResponseEntity<>(null, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    // get Question papers by student id for view papaer list
    @RequestMapping(value = "/getpapers/{id}", method = RequestMethod.GET)
    public ResponseEntity<List<PaperResultFinal>> getPapers(@PathVariable(value = "id") int sid) {
        try {
            List<PaperResultFinal> paperResultFinalList = questionServiceImpl.getPapers(sid);
            return new ResponseEntity<>(paperResultFinalList, HttpStatus.OK);
        } catch (Exception ex) {
            System.out.println(ex);
            return new ResponseEntity<>(null, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    // get Question paper by student id , paper number , subtopic  for view questions
    @RequestMapping(value = "/getpaper/{id}/", method = RequestMethod.GET)
    public ResponseEntity<QuestionResult> getPaper(@PathVariable(value = "id") int id) {
        try {
            QuestionResult questionResult = questionServiceImpl.getPaper(id);
            return new ResponseEntity<>(questionResult, HttpStatus.OK);
        } catch (Exception ex) {
            System.out.println(ex);
            return new ResponseEntity<>(null, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    // get Learning progress by student sid
    @RequestMapping(value = "/getprogress/{sid}/", method = RequestMethod.GET)
    public ResponseEntity<List<ProgressData>> getLearningProgress(@PathVariable(value = "sid") int sid) {
        try {
            List<ProgressData> ProgressDataList = questionServiceImpl.getLearningProgress(sid);
            return new ResponseEntity<>(ProgressDataList, HttpStatus.OK);
        } catch (Exception ex) {
            System.out.println(ex);
            return new ResponseEntity<>(null, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
}
